/* tests whether hard disk writes in-order */
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/fs.h>
#include <stdlib.h>

#define BLOCKSIZE 1024

typedef long Int;

int blkseek(int fd, int n)
{
	int ok;
#ifdef USE_LLSEEK
	loff_t pos;

	pos = (unsigned long long) n * (unsigned long long) BLOCKSIZE;
	ok = (-1 != llseek(fd, pos, SEEK_SET));
#else
	off_t pos;

	pos = (unsigned long) n * (unsigned long) BLOCKSIZE;
	ok = (-1 != lseek(fd, pos, SEEK_SET));
#endif
	return ok;
}


main(int argc, char *argv[])
{
  char *device;
  Int magic, skip;
  int fd;
  int size_blks;
  char *block=malloc(1024);
  Int i;

  if (argc != 4) {
    fprintf(stderr,"Usage: %s <device> <magic#> <skip>\n", argv[0]);
    exit(1);
  }
  device=argv[1];
  magic=atoi(argv[2]);
  skip=atoi(argv[3]);
  fd=open(device, O_SYNC|O_RDWR);
  if (fd<0) {
    perror("open");
    exit(1);
  }
  if (ioctl(fd, BLKGETSIZE, &size_blks) != 0) {
    perror("getting blocksize");
    exit(1);
  }
  size_blks /= 2;
  memset(block,0,BLOCKSIZE);
  ((Int *)block)[1] = magic;
  for (i=size_blks/2; i<size_blks; i += skip+1) {
    if (!blkseek(fd,i)) {
      perror("seek(i)");
      exit(1);
    }
    ((Int *)block)[0] = i;
    if (write(fd, block, BLOCKSIZE)!=BLOCKSIZE) {
      perror("write(i)");
      exit(1);
    }
    if (!blkseek(fd,0)) {
      perror("seek(0)");
      exit(1);
    }
    if (write(fd, block, BLOCKSIZE)!=BLOCKSIZE) {
      perror("write(0)");
      exit(1);
    }
  }
  exit(0);
}
